@@ -2,7 +2,7 @@
  * shmatch.c -- shell interface to posix regular expression matching.
  */
 
-/* Copyright (C) 2003 Free Software Foundation, Inc.
+/* Copyright (C) 2003-2015 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -64,7 +64,7 @@ sh_regmatch (string, pattern, flags)
 #endif
 
   rflags = REG_EXTENDED;
-  if (glob_ignore_case || match_ignore_case)
+  if (match_ignore_case)
     rflags |= REG_ICASE;
 #if !defined (ARRAY_VARS)
   rflags |= REG_NOSUB;
@@ -79,7 +79,8 @@ sh_regmatch (string, pattern, flags)
   matches = NULL;
 #endif
 
-  if (regexec (&regex, string, regex.re_nsub + 1, matches, 0))
+  /* man regexec: NULL PMATCH ignored if NMATCH == 0 */
+  if (regexec (&regex, string, matches ? regex.re_nsub + 1 : 0, matches, 0))
     result = EXECUTION_FAILURE;
   else
     result = EXECUTION_SUCCESS;		/* match */
@@ -91,11 +92,11 @@ sh_regmatch (string, pattern, flags)
   /* Store the parenthesized subexpressions in the array BASH_REMATCH.
      Element 0 is the portion that matched the entire regexp.  Element 1
      is the part that matched the first subexpression, and so on. */
-  unbind_variable ("BASH_REMATCH");
+  unbind_variable_noref ("BASH_REMATCH");
   rematch = make_new_array_variable ("BASH_REMATCH");
   amatch = array_cell (rematch);
 
-  if ((flags & SHMAT_SUBEXP) && result == EXECUTION_SUCCESS && subexp_str)
+  if (matches && (flags & SHMAT_SUBEXP) && result == EXECUTION_SUCCESS && subexp_str)
     {
       for (subexp_ind = 0; subexp_ind <= regex.re_nsub; subexp_ind++)
 	{
@@ -106,7 +107,9 @@ sh_regmatch (string, pattern, flags)
 	}
     }
 
+#if 0
   VSETATTR (rematch, att_readonly);
+#endif
 
   free (subexp_str);
   free (matches);
