@@ -25,6 +25,13 @@
 #include <sys/types.h>
 #include <posixtime.h>
 
+#include <bashintl.h>
+#include <stdc.h>
+
+#ifndef locale_decpoint
+extern int locale_decpoint PARAMS((void));
+#endif
+
 #include <stdio.h>
 
 struct timeval *
@@ -60,6 +67,32 @@ addtimeval (d, t1, t2)
   return d;
 }
 
+struct timeval *
+multimeval (d, m)
+     struct timeval *d;
+     int m;
+{
+  time_t t;
+
+  t = d->tv_usec * m;
+  d->tv_sec = d->tv_sec * m + t / 1000000;
+  d->tv_usec = t % 1000000;
+  return d;
+}
+
+struct timeval *
+divtimeval (d, m)
+     struct timeval *d;
+     int m;
+{
+  time_t t;
+
+  t = d->tv_sec;
+  d->tv_sec = t / m;
+  d->tv_usec = (d->tv_usec + 1000000 * (t % m)) / m;
+  return d;
+}
+  
 /* Do "cpu = ((user + sys) * 10000) / real;" with timevals.
    Barely-tested code from Deven T. Corzine <deven@ties.org>. */
 int
@@ -140,6 +173,7 @@ print_timeval (fp, tvp)
   minutes = timestamp / 60;
   seconds = timestamp % 60;
 
-  fprintf (fp, "%ldm%d.%03ds",  minutes, seconds, seconds_fraction);
+  fprintf (fp, "%ldm%d%c%03ds",  minutes, seconds, locale_decpoint (), seconds_fraction);
 }
+
 #endif /* HAVE_TIMEVAL */
