@@ -1,6 +1,6 @@
 /* shell.h -- The data structures used by the shell */
 
-/* Copyright (C) 1993-2009 Free Software Foundation, Inc.
+/* Copyright (C) 1993-2020 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -213,13 +213,17 @@ typedef struct _sh_input_line_state_t {
   size_t input_line_index;
   size_t input_line_size;
   size_t input_line_len;
+#if defined (HANDLE_MULTIBYTE)
+  char *input_property;
+  size_t input_propsize;
+#endif
 } sh_input_line_state_t;
 
 /* Let's try declaring these here. */
-extern char *parser_remaining_input __P((void));
+extern char *parser_remaining_input PARAMS((void));
 
-extern sh_parser_state_t *save_parser_state __P((sh_parser_state_t *));
-extern void restore_parser_state __P((sh_parser_state_t *));
+extern sh_parser_state_t *save_parser_state PARAMS((sh_parser_state_t *));
+extern void restore_parser_state PARAMS((sh_parser_state_t *));
 
-extern sh_input_line_state_t *save_input_line_state __P((sh_input_line_state_t *));
-extern void restore_input_line_state __P((sh_input_line_state_t *));
+extern sh_input_line_state_t *save_input_line_state PARAMS((sh_input_line_state_t *));
+extern void restore_input_line_state PARAMS((sh_input_line_state_t *));
